@@ -34,10 +34,11 @@ G_BEGIN_DECLS
 /* device constants */
 #define	CH_USB_VID				0x273f
 #define	CH_USB_PID_BOOTLOADER			0x1000
+#define	CH_USB_PID_BOOTLOADER2			0x1005
+#define	CH_USB_PID_BOOTLOADER_PLUS		0x1003
 #define	CH_USB_PID_FIRMWARE			0x1001
 #define	CH_USB_PID_FIRMWARE2			0x1004
 #define	CH_USB_PID_FIRMWARE_PLUS		0x1002
-#define	CH_USB_PID_BOOTLOADER_PLUS		0x1003
 #define	CH_USB_CONFIG				0x0001
 #define	CH_USB_INTERFACE			0x0000
 #define	CH_USB_HID_EP				0x0001
@@ -1229,6 +1230,9 @@ G_BEGIN_DECLS
 
 /* secret code */
 #define	CH_WRITE_EEPROM_MAGIC			"Un1c0rn2"
+#define	CH_FIRMWARE_ID_TOKEN1			"40338ceb"
+#define	CH_FIRMWARE_ID_TOKEN2			"2082b5e0"
+#define	CH_FIRMWARE_ID_TOKEN_PLUS		"6d6f05a9"
 
 /* input and output buffer offsets */
 #define	CH_BUFFER_INPUT_CMD			0x00
@@ -1362,6 +1366,7 @@ typedef enum {
 	CH_DEVICE_MODE_BOOTLOADER_PLUS,
 	CH_DEVICE_MODE_FIRMWARE_PLUS,
 	CH_DEVICE_MODE_FIRMWARE2,	/* since 1.2.2 */
+	CH_DEVICE_MODE_BOOTLOADER2,	/* since 1.2.3 */
 	CH_DEVICE_MODE_LAST
 } ChDeviceMode;
 
@@ -1372,6 +1377,8 @@ const gchar	*ch_multiplier_to_string	(ChFreqScale	 multiplier);
 const gchar	*ch_color_select_to_string	(ChColorSelect	 color_select);
 const gchar	*ch_measure_mode_to_string	(ChMeasureMode	 measure_mode);
 const gchar	*ch_device_mode_to_string	(ChDeviceMode	 device_mode);
+ChDeviceMode	 ch_device_mode_from_firmware	(const guint8	*data,
+						 gsize		 data_len);
 
 G_END_DECLS
 
