@@ -26,6 +26,7 @@
 #include <sqlite3.h>
 #include <syslog.h>
 
+#include "cd-cleanup.h"
 #include "cd-common.h"
 #include "cd-mapping-db.h"
 
@@ -70,32 +71,29 @@ cd_mapping_db_load (CdMappingDb *mdb,
 		    GError  **error)
 {
 	const gchar *statement;
-	gboolean ret = TRUE;
 	gchar *error_msg = NULL;
-	gchar *path = NULL;
 	gint rc;
+	_cleanup_free_ gchar *path = NULL;
 
 	g_return_val_if_fail (CD_IS_MAPPING_DB (mdb), FALSE);
 	g_return_val_if_fail (mdb->priv->db == NULL, FALSE);
 
 	/* ensure the path exists */
 	path = g_path_get_dirname (filename);
-	ret = cd_main_mkdir_with_parents (path, error);
-	if (!ret)
-		goto out;
+	if (!cd_main_mkdir_with_parents (path, error))
+		return FALSE;
 
 	g_debug ("CdMappingDb: trying to open database '%s'", filename);
 	syslog (LOG_INFO, "Using mapping database file %s", filename);
 	rc = sqlite3_open (filename, &mdb->priv->db);
 	if (rc != SQLITE_OK) {
-		ret = FALSE;
 		g_set_error (error,
 			     CD_CLIENT_ERROR,
 			     CD_CLIENT_ERROR_INTERNAL,
 			     "Can't open database: %s\n",
 			     sqlite3_errmsg (mdb->priv->db));
 		sqlite3_close (mdb->priv->db);
-		goto out;
+		return FALSE;
 	}
 
 	/* we don't need to keep doing fsync */
@@ -153,7 +151,7 @@ cd_mapping_db_load (CdMappingDb *mdb,
 				     "Failed to migrate mappings: SQL error: %s",
 				     error_msg);
 			sqlite3_free (error_msg);
-			goto out;
+			return FALSE;
 		}
 
 		/* remove old table data */
@@ -161,19 +159,16 @@ cd_mapping_db_load (CdMappingDb *mdb,
 		rc = sqlite3_exec (mdb->priv->db, statement,
 				   NULL, NULL, &error_msg);
 		if (rc != SQLITE_OK) {
-			ret = FALSE;
 			g_set_error (error,
 				     CD_CLIENT_ERROR,
 				     CD_CLIENT_ERROR_INTERNAL,
 				     "Failed to migrate mappings: SQL error: %s",
 				     error_msg);
 			sqlite3_free (error_msg);
-			goto out;
+			return FALSE;
 		}
 	}
-out:
-	g_free (path);
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -184,7 +179,6 @@ cd_mapping_db_empty (CdMappingDb *mdb,
 		     GError  **error)
 {
 	const gchar *statement;
-	gboolean ret = TRUE;
 	gchar *error_msg = NULL;
 	gint rc;
 
@@ -195,17 +189,15 @@ cd_mapping_db_empty (CdMappingDb *mdb,
 	rc = sqlite3_exec (mdb->priv->db, statement,
 			   NULL, NULL, &error_msg);
 	if (rc != SQLITE_OK) {
-		ret = FALSE;
 		g_set_error (error,
 			     CD_CLIENT_ERROR,
 			     CD_CLIENT_ERROR_INTERNAL,
 			     "SQL error: %s",
 			     error_msg);
 		sqlite3_free (error_msg);
-		goto out;
+		return FALSE;
 	}
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -369,7 +361,7 @@ cd_mapping_db_get_profiles (CdMappingDb *mdb,
 	gchar *statement;
 	gint rc;
 	GPtrArray *array = NULL;
-	GPtrArray *array_tmp = NULL;
+	_cleanup_ptrarray_unref_ GPtrArray *array_tmp = NULL;
 
 	g_return_val_if_fail (CD_IS_MAPPING_DB (mdb), NULL);
 	g_return_val_if_fail (mdb->priv->db != NULL, NULL);
@@ -399,7 +391,6 @@ cd_mapping_db_get_profiles (CdMappingDb *mdb,
 	/* success */
 	array = g_ptr_array_ref (array_tmp);
 out:
-	g_ptr_array_unref (array_tmp);
 	sqlite3_free (statement);
 	return array;
 }
@@ -419,7 +410,7 @@ cd_mapping_db_get_devices (CdMappingDb *mdb,
 	gchar *statement;
 	gint rc;
 	GPtrArray *array = NULL;
-	GPtrArray *array_tmp = NULL;
+	_cleanup_ptrarray_unref_ GPtrArray *array_tmp = NULL;
 
 	g_return_val_if_fail (CD_IS_MAPPING_DB (mdb), NULL);
 	g_return_val_if_fail (mdb->priv->db != NULL, NULL);
@@ -449,7 +440,6 @@ cd_mapping_db_get_devices (CdMappingDb *mdb,
 	/* success */
 	array = g_ptr_array_ref (array_tmp);
 out:
-	g_ptr_array_unref (array_tmp);
 	sqlite3_free (statement);
 	return array;
 }
